/*
 * csunits.h - ChipSound Voice Unit API
 *
 * Copyright (C) 2010-2012 David Olofson <david@olofson.net>
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 */

#ifndef CSUNITS_H
#define CSUNITS_H

#include "chipsound.h"

typedef struct CS_unit CS_unit;
typedef struct CS_cregdesc CS_cregdesc;

/* Control register descriptor */
struct CS_cregdesc
{
	const char	*name;		/* Register name for the compiler */
#if 0
	void(*write)(void *ud)
	CS_write_cb	write;		/* Callback to write control register */
	CS_read_cb	read;		/* Callback to read control register */
#endif
};

/* Unit descriptor */
struct CS_unit
{
	CS_otypes	type;		/* Object type (set by ChipSound) */
	const char	*name;		/* Unit name for 'uses' directive */
	int		nregisters;	/* Number of control registers */
	CS_cregdesc	*registers;	/* Array of register descriptors */
	int		channelsin;	/* Number of audio input channels */
	int		channelsout;	/* Number of audio output channels */
	unsigned	instancesize;	/* Size of instance data (bytes) */

	/* */
	void *(*instantiate)(CS_unit *u, void *imem);
};


CS_errors cs_RegisterUnit(CS_unit *u);

#endif	/* CSUNITS_H */
