/*
---------------------------------------------------------------------------
	a_limiter.h - Simple limiter
---------------------------------------------------------------------------
 * Copyright (C) 2001, 2002, 2009 David Olofson <david@olofson.net>
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 */

#include <stdlib.h>
#include "a_limiter.h"

int lim_open(LIM_limiter *lim, int samplerate)
{
	lim->samplerate = samplerate;
	lim_control(lim, LIM_THRESHOLD, 32768);
	lim_control(lim, LIM_RELEASE, 300000);
	lim->peak = (unsigned)(32768<<8);
	lim->attenuation = 0;
	return 0;
}

void lim_close(LIM_limiter *lim)
{
	/* Nothing to destruct */
}

void lim_control(LIM_limiter *lim, int param, int value)
{
	switch(param)
	{
	  case LIM_THRESHOLD:
		lim->threshold = (unsigned)(value << 8);
		if(lim->threshold < 256)
			lim->threshold = 256;
		break;
	  case LIM_RELEASE:
		if(lim->samplerate < 1)
			lim->samplerate = 44100;
		lim->release = (value << 8) / lim->samplerate;
		break;
	}
}

void lims_process(LIM_limiter *lim, int *in, int *out, unsigned frames)
{
	unsigned i;
	frames <<= 1;
	for(i = 0; i < frames; i += 2)
	{
		int gain;
		int lp = abs(in[i]) << 8;
		int rp = abs(in[i+1]) << 8;
		unsigned maxp = (unsigned)(lp > rp ? lp : rp);
		if(maxp > lim->peak)
		{
			lim->peak = maxp;
			gain = (32767<<16) / ((lim->peak+255)>>8);
		}
		else
		{
			gain = (32767<<16) / ((lim->peak+255)>>8);
			lim->peak -= lim->release;
			if(lim->peak < lim->threshold)
				lim->peak = lim->threshold;
		}
#ifdef A_USE_INT64
		out[i] = (Sint64)in[i] * (Sint64)gain >> 16;
		out[i+1] = (Sint64)in[i+1] * (Sint64)gain >> 16;
#else
		if(gain < 8192)
		{
			out[i] = in[i] * gain >> 16;
			out[i+1] = in[i+1] * gain >> 16;
		}
		else
		{
			out[i] = in[i] * (gain>>4) >> 12;
			out[i+1] = in[i+1] * (gain>>4) >> 12;
		}
#endif
	}
	lim->attenuation = (lim->peak - lim->threshold) >> 8;
}

void limss_process(LIM_limiter *lim, int *in, int *out, unsigned frames)
{
	unsigned i;
	frames <<= 1;
	for(i = 0; i < frames; i += 2)
	{
		int gain;
		int lp = abs(in[i]);
		int rp = abs(in[i+1]);
		unsigned maxp = (unsigned)(lp > rp ? lp : rp);
		unsigned p = maxp << 8;
		p += (maxp - abs(lp - rp)) << 7;
		if(p > lim->peak)
		{
			lim->peak = p;
			gain = (32767<<16) / ((lim->peak+255)>>8);
		}
		else
		{
			gain = (32767<<16) / ((lim->peak+255)>>8);
			lim->peak -= lim->release;
			if(lim->peak < lim->threshold)
				lim->peak = lim->threshold;
		}
#ifdef A_USE_INT64
		out[i] = (Sint64)in[i] * (Sint64)gain >> 16;
		out[i+1] = (Sint64)in[i+1] * (Sint64)gain >> 16;
#else
		if(gain < 8192)
		{
			out[i] = in[i] * gain >> 16;
			out[i+1] = in[i+1] * gain >> 16;
		}
		else
		{
			out[i] = in[i] * (gain>>4) >> 12;
			out[i+1] = in[i+1] * (gain>>4) >> 12;
		}
#endif
	}
	lim->attenuation = (lim->peak - lim->threshold) >> 8;
}
