/*
---------------------------------------------------------------------------
	a_limiter.h - Simple limiter
---------------------------------------------------------------------------
 * Copyright (C) 2001, 2002, 2009 David Olofson <david@olofson.net>
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 */

#define A_USE_INT64
#include "SDL.h"	/* For Sint64 */

#ifndef ADY_LIMITER_H
#define ADY_LIMITER_H

#define	DEFAULT_LIM_RELEASE	(65536 * 1 / 44100)
#define	DEFAULT_LIM_THRESHOLD	32768

typedef struct LIM_limiter
{
	int		samplerate;
	unsigned	threshold;	/* Reaction threshold */
	int		release;	/* Release "speed" */
	unsigned	peak;		/* Filtered peak value */
	unsigned	attenuation;	/* Current output attenuation */
} LIM_limiter;

enum LIM_params
{
	LIM_THRESHOLD = 0,	/* 16 bit sample units */
	LIM_RELEASE		/* Units per second */
};


int lim_open(LIM_limiter *lim, int samplerate);
void lim_close(LIM_limiter *lim);

void lim_control(LIM_limiter *lim, int param, int value);

/*
 * Limiter stereo process function.
 * Supports in-place processing.
 */
void lims_process(LIM_limiter *lim, int *in, int *out, unsigned frames);

/*
 * Smart Stereo version.
 *
 * This algorithm takes both channels in account in a way
 * that reduces the effect of the center appearing to have
 * more power after compression of signals with unbalanced
 * stereo images.
 *
 * A dead center signal can only get 3 dB louder than the
 * same signal in one channel only, as opposed to the
 * normal 6 dB of a limiter that only looks at max(L, R).
 *
 * Meanwhile, with "normal" material (ie most power
 * relatively centered), this limiter gets an extra 3 dB
 * compared to a limiter that checks (L+R).
 */
void limss_process(LIM_limiter *lim, int *in, int *out, unsigned frames);

#endif /*ADY_LIMITER_H*/
